import { badgeVariants } from "@/components/ui/badge.tsx";
import { Link } from "wouter";
import { cn } from "@/lib/utils.ts";
import { cva } from "class-variance-authority"

enum TagKind {
    Default = "default",
    Type = "type",
    Character = "character",
    Series = "series",
    Group = "group",
    Artist = "artist",
    Male = "male",
    Female = "female",
}

type TagKindType = `${TagKind}`;

export function getTagKind(tagname: string): TagKindType {
    if (tagname.match(":") === null) {
        return "default";
    }
    const prefix = tagname.split(":")[0];
    return prefix as TagKindType;
}

export function toPrettyTagname(tagname: string): string {
    const kind = getTagKind(tagname);
    const name = tagname.slice(kind.length + 1);

    switch (kind) {
        case "male":
            return `♂ ${name}`;
        case "female":
            return `♀ ${name}`;
        case "artist":
            return `🎨 ${name}`;
        case "group":
            return `🖿 ${name}`;
        case "series":
            return `📚 ${name}`
        case "character":
            return `👤 ${name}`;
        case "default":
            return tagname;
        default:
            return name;
    }
}

interface TagBadgeProps {
    tagname: string;
    className?: string;
    disabled?: boolean;
}


export const tagBadgeVariants = cva(
    cn(badgeVariants({ variant: "default"}), "px-1"),
    {
        variants: {
            variant: {
                default: "bg-[#4a5568] hover:bg-[#718096]",
                type: "bg-[#d53f8c] hover:bg-[#e24996]",
                character: "bg-[#52952c] hover:bg-[#6cc24a]",
                series: "bg-[#dc8f09] hover:bg-[#e69d17]",
                group: "bg-[#805ad5] hover:bg-[#8b5cd6]",
                artist: "bg-[#319795] hover:bg-[#38a89d]",
                female: "bg-[#c21f58] hover:bg-[#db2d67]",
                male: "bg-[#2a7bbf] hover:bg-[#3091e7]",
            },
        },
        defaultVariants: {
            variant: "default",
        },
    }
);

export default function TagBadge(props: TagBadgeProps) {
    const { tagname } = props;
    const kind = getTagKind(tagname);
    return <li className={
        cn( tagBadgeVariants({ variant: kind }),
            props.disabled && "opacity-50",
            props.className,
        )
    }><Link to={props.disabled ? '': `/search?allow_tag=${tagname}`}>{toPrettyTagname(tagname)}</Link></li>;
}