import { z } from "zod";
export { ZodError } from "zod";

export const DocumentBodySchema = z.object({
	title: z.string(),
	content_type: z.string(),
	basepath: z.string(),
	filename: z.string(),
	modified_at: z.number(),
	content_hash: z.string(),
	additional: z.record(z.unknown()),
	tags: z.array(z.string()),
	pagenum: z.number().int(),
});

export type DocumentBody = z.infer<typeof DocumentBodySchema>;

export const DocumentSchema = DocumentBodySchema.extend({
	id: z.number(),
	created_at: z.number(),
	deleted_at: z.number().nullable(),
});

export type Document = z.infer<typeof DocumentSchema>;

export const TagSchema = z.object({
	name: z.string(),
	description: z.string().nullable(),
});

export type Tag = z.infer<typeof TagSchema>;

export const TagRelationSchema = z.object({
	doc_id: z.number(),
	tag_name: z.string(),
});

export type TagRelation = z.infer<typeof TagRelationSchema>;

export const PermissionSchema = z.object({
	username: z.string(),
	name: z.string(),
});

export type Permission = z.infer<typeof PermissionSchema>;

export const UserSchema = z.object({
	username: z.string(),
	password_hash: z.string(),
	password_salt: z.string(),
});

export type User = z.infer<typeof UserSchema>;

export const SchemaMigrationSchema = z.object({
	version: z.string().nullable(),
	dirty: z.boolean(),
});

export type SchemaMigration = z.infer<typeof SchemaMigrationSchema>;



export const QueryListOptionSchema = z.object({
	/**
	 * search word
	 */
	word: z.string().optional(),
	allow_tag: z.array(z.string()).default([]),
	limit: z.number().default(20),
	use_offset: z.boolean().default(false),
	cursor: z.number().optional(),
	offset: z.number().optional(),
	eager_loading: z.boolean().default(true),
	content_type: z.string().optional(),
});

export type QueryListOption = z.infer<typeof QueryListOptionSchema>;

export const LoginRequestSchema = z.object({
	username: z.string(),
	password: z.string(),
});

export type LoginRequest = z.infer<typeof LoginRequestSchema>;

export const LoginResetRequestSchema = z.object({
	username: z.string(),
	oldpassword: z.string(),
	newpassword: z.string(),
});

export type LoginResetRequest = z.infer<typeof LoginResetRequestSchema>;