import event from "node:events";
import { type FSWatcher, promises, watch } from "node:fs";
import { join } from "node:path";
import type { DocumentAccessor } from "../../model/doc.ts";
import type { DiffWatcherEvent, IDiffWatcher } from "../watcher.ts";
import { setupHelp } from "./util.ts";

const { readdir } = promises;

export class CommonDiffWatcher extends event.EventEmitter implements IDiffWatcher {
	on<U extends keyof DiffWatcherEvent>(event: U, listener: DiffWatcherEvent[U]): this {
		return super.on(event, listener);
	}
	emit<U extends keyof DiffWatcherEvent>(event: U, ...arg: Parameters<DiffWatcherEvent[U]>): boolean {
		return super.emit(event, ...arg);
	}
	private _path: string;
	private _watcher: FSWatcher;

	constructor(path: string) {
		super();
		this._path = path;
		this._watcher = watch(this._path, { persistent: true, recursive: false },
			async (eventType, filename) => {
				// if filename is null, it means the path is not exist
				if (filename === null) return;
				if (eventType === "rename") {
					const cur = await readdir(this._path);
					// add
					if (cur.includes(filename)) {
						this.emit("create", join(this.path, filename));
					} else {
						this.emit("delete", join(this.path, filename));
					}
				}
			});
	}
	async setup(cntr: DocumentAccessor): Promise<void> {
		await setupHelp(this, this.path, cntr);
	}
	public get path() {
		return this._path;
	}
	watchClose() {
		this._watcher.close();
	}
}
